/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#include <RTPriv/OSEK_VDX.h>
#include <RTPriv/Mutex.h>
#include <RTPriv/RTstdio.h>
#include <RTPriv/SyncObj.h>
#include <RTPriv/Task.h>

#include <RTPubl/Main.h>

extern int RTMain_mainLine_finish( void );

RTTask RTTasks[ RT_OSEK_NUM_TASKS ];

#if USE_THREADS
static RTMutex mutex;
#endif

#if USE_THREADS
static RTAction mainFinishAction;
#endif

/*
 * initRTTasks initializes the RTTasks array.
 */

static void initRTTasks( void )
{
    RTTasks[ 0 ]._id   = RTTask0;
    RTTasks[ 0 ]._prio = 0;
#if USE_THREADS
    RTTasks[ 1 ]._id   = RTTask1;
    RTTasks[ 1 ]._prio = 1;
    RTTasks[ 2 ]._id   = RTTask2;
    RTTasks[ 2 ]._prio = 2;
    RTTasks[ 3 ]._id   = RTTask3;
    RTTasks[ 3 ]._prio = 3;
    RTTasks[ 4 ]._id   = RTTask4;
    RTTasks[ 4 ]._prio = 4;
    RTTasks[ 5 ]._id   = RTTask5;
    RTTasks[ 5 ]._prio = 5;
    RTTasks[ 6 ]._id   = RTTask6;
    RTTasks[ 6 ]._prio = 6;
    RTTasks[ 7 ]._id   = RTTask7;
    RTTasks[ 7 ]._prio = 7;
#endif
}

#if USE_THREADS
void RTMain_finishFunc( void )
{
    unsigned int i;

    mainFinishAction.exec = (void( * )( void * ))RTMain_mainLine_finish;        
    mainFinishAction.arg = (void *)0;

    /* Put the cleanup action on the lowest priority task. This way all the
       controllers will be able to finish whatever they are doing. */
    RTMutex_enter( &mutex );

    /* Empty the queues. */ 
    for( i = 0U; i < RT_OSEK_NUM_TASKS; ++i )
    {
	RTTasks[ i ].head  = (RTAction *)0;
	RTTasks[ i ].tail  = (RTAction *)0;
    }

    RTTask_enqueueAction( &RTTasks[  0 ], &mainFinishAction );
    RTTask_lockQueues(); /* Make sure no more actions can get enqueued. */
    RTMutex_leave( &mutex );
    ActivateTask( RTTasks[  0 ]._id );
}
#endif

/*
 * Each OSEK task can have several RTThreads of the same priority.
 */

static void taskBody( unsigned int prio )
{
    /*
     * Important:
     * For performance reasons, here is assumpion that the array index is equal
     * to task priority.
     */
    RTTask   * task = &RTTasks[ prio ];
    RTAction * action;

    for( ;; )
    {
#if USE_THREADS
	RTMutex_enter( &mutex );
#endif

	action = RTTask_dequeueAction( task );

#if USE_THREADS
	RTMutex_leave( &mutex );
#endif

	if( ! action )
	    break;

	( *action->exec )( action->arg );
    }

    TerminateTask();   /* Go to sleep. This replaces RTSyncObject_wait() . */
}

/*
 * Here are the startup functions for tasks at each priority. Comment out any
 * functions for unused priorities.
 */

/*
 * The execution begins here.
 */

void RTTask0func( void )
{
    static short firstTime = 1;

    if( firstTime )
    {
	unsigned int i;

        initRTTasks();

	/* Make sure action queues are empty. */ 
	for( i = 0U; i < RT_OSEK_NUM_TASKS; ++i )
	{
	    RTTasks[ i ].head  = (RTAction *)0;
	    RTTasks[ i ].tail  = (RTAction *)0;
	}

	(void)RTMain_entryPoint( 0, (const char * const *)0 );

	firstTime = 0;
    }

    taskBody( 0U );
}

#if USE_THREADS

void RTTask1func( void )
{
    taskBody( 1U );
}

void RTTask2func( void )
{
    taskBody( 2U );
}

void RTTask3func( void )
{
    taskBody( 3U );
}

void RTTask4func( void )
{
    taskBody( 4U );
}

void RTTask5func( void )
{
    taskBody( 5U );
}

void RTTask6func( void )
{
    taskBody( 6U );
}

void RTTask7func( void )
{
    taskBody( 7U );
}

#endif

/*
 * The OS specific versions of the hooks should be defined in Main/hooks.c in
 * the OS's src/target.
 */
extern void StartupHook_OS( void );
extern void ShutdownHook_OS( void );

void StartupHook( void )
{
    StartupHook_OS();
}

StatusType shutdownError;

void ShutdownHook( StatusType error )
{
    ShutdownHook_OS();

    shutdownError = error;
}

void ErrorHandler( void )
{
    ShutdownOS( 0xffff );
}
