/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#include <RTPubl/SyncObj.h>
#include <RTPubl/Timespec.h>

#include <RTPriv/RTstdio.h>
#include <RTPriv/SyncObj.h>

extern RTTask RTTasks[ RT_OSEK_NUM_TASKS ];

static RTMutex mutex;

int RTSyncObject_timedwait( RTSyncObject     * this,
			    const RTTimespec * expireTime )
{
    long delta = RTTimespec_ticksFromNow( expireTime, 1000000L );

    if( delta > 0L )
    {
	RTMutex_enter( &mutex );

	switch( this->_state )
	{
	    case Waiting:
	    case TimedWaiting:
		/* Should not get here. */
		break;

	    case Signalled:
		/* There already is an action in the queue. */
		this->_state = NotSignalled;
		break;

	    case NotSignalled:
		this->_action.exec = (void( * )( void * ))RTSyncObject_signal;
		this->_action.arg = (void *)this;

		if( SetRelAlarm( RTTimerAlarm, (TickType)delta, (TickType)0 ) != E_OK )
		{
#if STDIO_ENABLED
		    RTStdio_panic( "SetRelAlarm" );
#endif
		}

		/* The highest priority task will be awakened by the alarm.
	   	Tell it to signal our controller. */
		RTTask_enqueueAction( &RTTasks[ RT_OSEK_NUM_TASKS - 1 ],
				      &this->_action );

		this->_state = TimedWaiting;

		break;
	}

	RTMutex_leave( &mutex );
    }

    return 0;
}
